"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.calendarMonth = exports.getNextDay = exports.getPreviousDay = exports.getNextMonth = exports.getPreviousMonth = exports.getDaysInMonth = void 0;
const isLeap = (year) => year % 400 === 0 || (year % 100 !== 0 && year % 4 === 0);
exports.getDaysInMonth = (year, month) => {
    const daysInMonth = [31, -1, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
    if (month === 2 /* February */) {
        return isLeap(year) ? 29 : 28;
    }
    return daysInMonth[month - 1];
};
exports.getPreviousMonth = (year, month) => {
    if (month === 1 /* January */) {
        return {
            month: 12 /* December */,
            year: year - 1
        };
    }
    return {
        month: month - 1,
        year
    };
};
exports.getNextMonth = (year, month) => {
    if (month === 12 /* December */) {
        return {
            month: 1,
            year: year + 1
        };
    }
    return {
        month: month + 1,
        year
    };
};
exports.getPreviousDay = (year, month, day) => {
    if (day === 1) {
        const prevMonth = exports.getPreviousMonth(year, month);
        const lastDayInPrevMonth = exports.getDaysInMonth(prevMonth.year, prevMonth.month);
        const previousDayFromPreviousMonth = new Date(Date.UTC(prevMonth.year, prevMonth.month - 1, lastDayInPrevMonth));
        return {
            dayInMonth: previousDayFromPreviousMonth.getUTCDate(),
            dayInWeek: previousDayFromPreviousMonth.getUTCDay(),
            month: prevMonth
        };
    }
    const previousDay = new Date(Date.UTC(year, month - 1, day - 1));
    return {
        dayInMonth: previousDay.getUTCDate(),
        dayInWeek: previousDay.getUTCDay(),
        month: { month, year }
    };
};
exports.getNextDay = (year, month, day) => {
    const isLastDayInMonth = (y, m, d) => exports.getDaysInMonth(y, m) === d;
    if (isLastDayInMonth(year, month, day)) {
        const nextMonth = exports.getNextMonth(year, month);
        const nextDayOfNextMonth = new Date(Date.UTC(nextMonth.year, nextMonth.month - 1, 1));
        return {
            dayInMonth: nextDayOfNextMonth.getUTCDate(),
            dayInWeek: nextDayOfNextMonth.getUTCDay(),
            month: nextMonth
        };
    }
    const nextDay = new Date(Date.UTC(year, month - 1, day + 1));
    return {
        dayInMonth: nextDay.getUTCDate(),
        dayInWeek: nextDay.getUTCDay(),
        month: { month, year }
    };
};
/**
 * Get calendar for given month.
 *
 * @param {number} year - year [1900 – 2100]
 * @param {number} month - Month [1  - 12]
 * @param {number} [startOfTheWeek=0] - Start of the week [0 – 6] where 0 is Sunday, and 6 is Saturday
 *
 * @returns {Object[][]}
 */
function calendarMonth(year, month, startOfTheWeek = 0 /* Sunday */) {
    const firstDayInMonth = new Date(Date.UTC(year, month - 1, 1));
    let startingDay = {
        dayInMonth: firstDayInMonth.getUTCDate(),
        dayInWeek: firstDayInMonth.getUTCDay(),
        month: { month, year }
    };
    // go back to first start of the week (Monday or Sunday, or any other really)
    if (startingDay.dayInWeek !== startOfTheWeek) {
        let previousDay = exports.getPreviousDay(startingDay.month.year, startingDay.month.month, startingDay.dayInMonth);
        while (previousDay.dayInWeek !== startOfTheWeek) {
            previousDay = exports.getPreviousDay(previousDay.month.year, previousDay.month.month, previousDay.dayInMonth);
        }
        startingDay = previousDay;
    }
    const daysInWeek = 7;
    const weeksInCalendarMonth = 6;
    const calendarMonthDays = daysInWeek * weeksInCalendarMonth;
    const days = [];
    let currentDay = startingDay;
    // then go next day up until all 42 (6 weeks) are filled
    while (days.length < calendarMonthDays) {
        days.push(currentDay);
        currentDay = exports.getNextDay(currentDay.month.year, currentDay.month.month, currentDay.dayInMonth);
    }
    return [
        days.slice(daysInWeek * 0, daysInWeek * 1),
        days.slice(daysInWeek * 1, daysInWeek * 2),
        days.slice(daysInWeek * 2, daysInWeek * 3),
        days.slice(daysInWeek * 3, daysInWeek * 4),
        days.slice(daysInWeek * 4, daysInWeek * 5),
        days.slice(daysInWeek * 5, daysInWeek * 6)
    ];
}
exports.calendarMonth = calendarMonth;
